import { Component, Input, OnInit, Output, EventEmitter, OnChanges, SimpleChanges, ViewChild, ViewContainerRef, ComponentFactoryResolver } from '@angular/core';
import { Router, NavigationStart, NavigationEnd, NavigationCancel, NavigationError, RoutesRecognized } from '@angular/router';
import { PinyinService } from '../pinyin.service';

@Component({
  selector: 'dc-auto-component',
  template: `
  <div class="display-flex width-100 flex-direction-column" [style.position]="'relative'" [style.max-width]="width">
    <input type="text" [(ngModel)]="text" (keydown)="press($event)" (blur)="blur()" (focus)="focus()" (ngModelChange)="changeText()">
    <div class="complete-container width-100 flex-direction-column" *ngIf="showContainer">
        <div *ngFor="let c of show_source" class="source-title" [ngClass]="{active:c.show}" (click)="selectText(c)">
            {{c.title}}
        </div> 
    </div>
</div>
  `,
  styles: [
    '.title.active{color: red}',
    '.title{cursor: pointer;}',
    '.complete-container{position: absolute;top:24px;background-color: wheat;}',
    '.source-title{max-width: 100%;overflow: hidden;text-overflow: ellipsis;cursor: pointer;}',
    '.source-title.active{color: rebeccapurple;}',
  ]
})
export class AutoCompleteComponent implements OnInit, OnChanges {

  showContainer: boolean;
  @Input() source;
  @Input() width;
  @Output() textChangeEvent = new EventEmitter();
  text: string;
  show_source: Array<any> = [];
  constructor(private resolver: ComponentFactoryResolver, private router: Router, private pinyinService: PinyinService) {

  }
  ngOnInit(): void {
  }
  ngOnChanges(changes: SimpleChanges): void {
  }
  blur() {
    this.showContainer = false;
  }
  focus() {
    this.changeText();
  }

  press(event) {
    if (event.key == 'Tab') {
      event.preventDefault();
      for (let i = 0; i < this.show_source.length; i++) {
        if (this.show_source[i].show) {
          this.show_source[i].show = false;
          if (i == this.show_source.length - 1) {
            this.show_source[0].show = true;
          } else {
            this.show_source[i + 1].show = true;
          }
          break;
        }
      }
    }

    if (event.key == 'Enter') {
      for (let _c of this.show_source) {
        if (_c.show) {
          this.text = _c.title;
        }
      }
      this.showContainer = false;
    }
  }
  changeText() {
    if (this.text == null) {
      this.text = "";
    }
    this.show_source = [];
    let tempSource: Array<any> = this.source.filter(item => {
      return item.indexOf(this.text) >= 0 || this.pinyinService.toPinyin(item).indexOf(this.text)>=0
    });

    for (let i = 0; i < tempSource.length; i++) {
      let _show = i == 0;
      this.show_source.push({
        title: tempSource[i],
        show: _show
      });
    }
    this.showContainer = true;
  }

  selectText(t) {
    this.text = t;
    this.showContainer = false;
  }
}
