import { Component, OnChanges, OnInit, DoCheck, OnDestroy, SimpleChanges, ComponentFactoryResolver, ViewChild, ViewContainerRef, HostListener, ElementRef, Injector, ComponentRef, ReflectiveInjector, ApplicationRef } from '@angular/core';
import { Router, NavigationStart, NavigationEnd, NavigationCancel, NavigationError, RoutesRecognized } from '@angular/router';

export interface CanDraw {
  beforeLeave(): boolean | Promise<boolean>;
  beforeClose(): void;
}


@Component({
  selector: "drawer",
  template: `
   <div class="draw-container" [ngClass]="{'show':show}">
    <ng-template #componentContainer></ng-template>
   </div>
  `,
  styles: [
    '.draw-container{'+
      'width: 66%;'+
      'position: absolute;'+
      'height: 100vh;'+
      'right: 0;'+
      'background-color: white;'+
      'max-width: 0;'+
      'transition: max-width 0.5s ease-in,color 0.3s ease-out;'+
      'box-shadow: -2px 0px 15px 0px #888888;'+
      'z-index: 1;'+
      'background: #fcfcfc;'+
      'padding: 30px;'+
      'overflow-x: hidden;'+
      'overflow-y: auto}',
      '.draw-container.show{max-width:100%;}'
  ]
})

export class DrawComponent implements OnChanges, OnInit, DoCheck, OnDestroy {
  instance: any;
  show;
  injectComponet;
  data;
  componentRef;
  beforeLeave;
  firstInit;
  closeResolve;
  @ViewChild("componentContainer", { read: ViewContainerRef }) container: ViewContainerRef;
  constructor(private router: Router, private resolver: ComponentFactoryResolver,
    private elementRef: ElementRef, private injector: Injector, private applicationRef: ApplicationRef) {
    this.show = false;
    this.firstInit = true;
  }

  @HostListener('document:click', ['$event'])
  onClick(event) {
    //第一次点击无效
    if (this.firstInit) {
      this.firstInit = false;
      return;
    }
    if (!this.elementRef.nativeElement.contains(event.target)) {
      let result = this.componentRef.instance.beforeLeave();
      if (typeof (result) == "boolean" && result) {
        this.closeComponent();
      }
      if (typeof (result) == "object" && result.then) {
        result.then(r => {
          if (r) {
            this.closeComponent();
          }
        })
      }
    }
  }

  private closeComponent() {
    this.show = false;
    setTimeout(() => {
      this.container.clear();
      this.closeResolve('close');
    }, 500);
  }

  ngAfterViewInit() {
    setTimeout(() => {
      this.show = true;
    });
  }
  ngOnDestroy(): void {
    this.container.clear();
  }
  ngDoCheck(): void {

  }
  ngOnInit(): void {
    this.container.clear();
    this.componentRef = this.container.createComponent(this.resolver.resolveComponentFactory(this.injectComponet));
    this.instance = this.componentRef.instance;
    this.instance.data = this.data;
  }
  ngOnChanges(changes: SimpleChanges): void {

  }
}
