import {
  Component,
  Input,
  Output,
  EventEmitter,
  ContentChildren,
  OnInit,
  QueryList,
  ContentChild,
  TemplateRef,
  OnChanges,
  SimpleChanges, ViewChild, HostListener
} from '@angular/core';
import { Router, NavigationStart, NavigationEnd, NavigationCancel, NavigationError, RoutesRecognized } from '@angular/router';
import { TableColumnComponent } from './column.component';

@Component({
  selector: 'dc-table',
  template: `
  <ng-template #filterTemplate let-header>
  <div *ngIf="header.filter.type == 'text'" class="filter-box">
    <p>过滤:</p>
    <p><input type="text" value="" (keyup.enter)="clickFilter($event, header)"></p>
  </div>
</ng-template>
<table class="table">
  <thead>
  <tr>
    <th *ngIf="checkBox">
      <input type='checkbox' [(ngModel)]='allSelect' (ngModelChange)="allSelectCheck()">
    </th>
    <th class="table-title" [style.width]="header.width" *ngFor="let header of headers" title="{{header.title}}">
      <div class="table-title-th text-over-flow">
        <div style="flex: 1;"[ngClass]="{canSort:header.canSort}">
        {{header.title}}
          <div
            [ngClass]="{'sort': (!nowSort.field && header.canSort) ||(nowSort.field==header.field && nowSort.sort=='asc'),
            'sort-desc':(nowSort.field==header.field && nowSort.sort=='desc')}" (click)="sort(header)">
          </div>
        </div>
        <div *ngIf="header.filter" class="table-th-filter">
          <i (click)="showFilterBox($event)">过滤</i>
          <ng-container [ngTemplateOutlet]="filterTemplate" [ngTemplateOutletContext]="{
            $implicit: header
          }"></ng-container>
        </div>
      </div>
    </th>
  </tr>
  </thead>

  <tbody>
  <tr *ngFor="let data of datas let i = index">
    <td *ngIf="checkBox">
      <input type='checkbox' [(ngModel)]='checks[i]' (ngModelChange)="checkSelect()">
    </td>
    <td *ngFor="let header of headers">
      <ng-container [ngTemplateOutlet]="template" [ngTemplateOutletContext]="{
                    $implicit: {
                      d: data,
                      h: header
                    }}"></ng-container>
    </td>
  </tr>
  </tbody>
</table>
  `,
  styles: [
    '.canSort {' +
    'cursor: pointer}',

    '.table-title {' +
    'font-size: 14px;}',

    'table {' +
    'border: none;}',

    'table th {' +
    'background-color: #fff;' +
    'border-bottom: 1px solid #cccccc;}',

    'table td {' +
    'border-bottom: 1px solid #e2e2e2;' +
    'border-top: none;}',

    '.sort {' +
    'background-image: url(/assets/img/sort.png);' +
    'background-repeat: no-repeat;' +
    'background-position: center;' +
    'width: 20px;' +
    'height: 13px;' +
    'display: inline-block;}',

    '.sort-desc {' +
    'background-image: url(/assets/img/sort.png);' +
    'background-repeat: no-repeat;' +
    'background-position: center;' +
    'width: 20px;' +
    'height: 13px;' +
    'transform: rotate(180deg);}',

    '.table-title-th {' +
    'display: flex;' +
    'text-align: center;' +
    /*justify-content: center;*/
    'line-height: 15px;}' +

    '.table {' +
    'font-size: 12px;' +
    'color: #666666 !important;' +
    '}',

    '.table th {' +
    'font-size: 14px;' +
    'color: #333333 !important;' +
    '}',

    'table-th-filter{' +
    'position: relative;' +
    '}',

    '.filter-box {' +
    'position: absolute;' +
    'display: none;' +
    'background: #fff;' +
    'border: 1px solid #ccc;' +
    'border-radius: 3px;' +
    'margin-top: 5px;' +
    'padding-bottom: 5px;' +
    '}',
  ]
})
export class TableComponent implements OnInit {
  @ContentChild(TemplateRef) template: TemplateRef<TableColumnComponent>;
  // @ContentChildren(TableColumnComponent)
  // childCmps: QueryList<TableColumnComponent>;
  @ViewChild('filterTemplate') filterTemplate;
  @Input() headers: Array<any>;
  _datas;
  @Input() set datas(v) {
    this.allSelect = false;
    this._datas = v;
    this.initCheck();
  }

  get datas() {
    return this._datas;
  }

  @Input() checkBox: boolean = false;
  @Output() sortEvent: EventEmitter<number> = new EventEmitter<number>();
  @Output() checkEvent: EventEmitter<any> = new EventEmitter<any>();
  nowSort: any = {};
  allSelect: boolean = false;
  checks: Array<boolean> = [];
  checks_data = [];

  constructor(private router: Router) {

  }

  ngOnInit(): void {
    // this.initCheck();
  }

  private initCheck() {
    this.checks = [];
    for (let d of this.datas) {
      this.checks.push(false);
    }
  }

  ngAfterContentInit() {
  }

  /*
	ngOnChanges(changes: SimpleChanges): void {
	  if (changes.datas && !changes.datas.firstChange) {
		this.allSelect = false;
		this.datas = changes.datas.currentValue;
		this.initCheck();
	  }
	}*/

  allSelectCheck() {
    this.checks = [];
    for (let d of this.datas) {
      this.checks.push(this.allSelect);
    }
    this.sendCheckEvent();
  }

  checkSelect() {
    let allTrue = true;
    for (let c of this.checks) {
      if (!c) {
        allTrue = false;
      }
    }

    if (allTrue) {
      this.allSelect = true;
    } else {
      this.allSelect = false;
    }
    this.sendCheckEvent();
  }

  private sendCheckEvent(): any {
    this.checks_data = [];
    for (let i = 0; i < this.datas.length; i++) {
      if (this.checks[i]) {
        this.checks_data.push(this.datas[i]);
      }
    }
    this.checkEvent.emit(this.checks_data);
  }

  sort(header) {
    if (!header.canSort) {
      return;
    }
    if (this.nowSort.field == header.field) {
      if (this.nowSort.sort == 'asc') {
        this.nowSort.sort = 'desc';
      } else {
        this.nowSort.sort = 'asc';
      }
    } else {
      this.nowSort.field = header.field;
      this.nowSort.sort = 'desc';
    }
    if (this.sortEvent) {
      this.sortEvent.emit(this.nowSort);
    }
  }

  showFilter = [];

  showFilterBox(event) {
    event.stopPropagation();
    event.preventDefault();
    let filterBoxDom = event.target.nextElementSibling;
    filterBoxDom.style.display = 'block';
    this.showFilter.push(filterBoxDom);
  }

  clickFilter(event, header) {
    header.filter.fn(event.target.value);
    let filterBoxDom = event.target.parentNode.parentNode;
    filterBoxDom.style.display = 'none';
  }

  @HostListener('document:click', ['$event'])
  onDocumentClick($event) {
    for (let i = 0; i < this.showFilter.length; i++) {

    }
  }
}
