import {
  Component,
  Input,
  OnInit,
  Output,
  EventEmitter,
  OnChanges,
  SimpleChanges,
  ViewChild,
  ViewContainerRef,
  ComponentFactoryResolver
} from '@angular/core';
import { Router, NavigationStart, NavigationEnd, NavigationCancel, NavigationError, RoutesRecognized } from '@angular/router';

@Component({
  selector: 'dc-node',
  template: `
  <div class="display-flex width-100 flex-direction-column node-container">
  <div class="display-flex width-100">
    <div class="before-node">
    </div>
    <div class="icon" [class.cursor-pointer]="nodeData[keyChild]!=null && nodeData[keyChild].length>0">
      <i (click)="changeShow()" *ngIf="nodeData[keyChild]!=null && nodeData[keyChild].length>0">{{getText()}}</i>
    </div>
    <div class="tree-icon" *ngIf="options && options['iconCls']" [ngClass]="[getFileIcon(),getFolderIcon(),getExpandIcon()]">
    </div>
    <div class="cursor-pointer" (click)="clickNode(nodeData)"
         [class.active]="nodeData.lighthight" [class.selected]="nodeData.selected">
      {{nodeData[keyName]}}
    </div>
    <div class="" *ngIf="options && options.actionBtn" style="float: right;">
      <ng-template ngFor let-actbtn [ngForOf]="options.actionBtn">
        <i *ngIf="(root && !actbtn.rootHide) || !root" class="{{actbtn.cls}}"
           (click)="doNodeFn(actbtn)">{{actbtn.text}}</i>
      </ng-template>
    </div>
  </div>

  <div *ngIf="nodeData[keyChild]!=null && nodeData[keyChild].length>0 && nodeData.show"
       class="display-flex width-100 flex-direction-column sub-node-container">
    <div *ngFor="let node of nodeData[keyChild]">
      <div>
        <dc-node [nodeData]="node" [keyChild]="keyChild" [keyId]="keyId"
                 [options]="options" [keyName]="keyName"
                 (clickEvent)="_clickNode($event)" (extendEvent)="_extendNode($event)"></dc-node>
      </div>
    </div>
  </div>
</div>
  `,
  styles: [
    '.sub-node-container{margin-left: 10px;}',
    '.before-node{}',
    '.left-line{position: absolute;width: 1px;height: 100000px;border-left: 1px solid red;}',
    '.node-container{overflow: hidden;position: relative;};',
    '.icon{width: 10px;}',
    '.noIcon{margin-left: 10px;}',
    '.tree-icon{display: inline-block;width: 20px;height: 20px;}'
  ]
})
export class NodeComponent implements OnInit, OnChanges {
  showSub: any;
  _nodeData;
  @Input() root: boolean;
  @Input() keyId: string;
  @Input() keyName: string;
  @Input() keyChild: string;
  @Input() options: any;
  @Output() clickEvent = new EventEmitter();
  @Output() extendEvent = new EventEmitter();
  iconClsFolder: string;
  iconClsExpand: string;
  iconClsFile: string;


  @Input() set nodeData(v) {
    this._nodeData = v;
    this.init();
  }

  get nodeData() {
    return this._nodeData;
  }

  @Output() nodeChangeEvent = new EventEmitter();


  constructor(private resolver: ComponentFactoryResolver, private router: Router) {

  }

  init(): any {
    if (this.needShow(this.nodeData)) {
      this.nodeData.show = true;
    }
  }

  private needShow(node: any): any {
    if (node.show) {
      return true;
    }
    if (node.subNode == null || node.subNode.length == 0) {
      return false;
    }
    let result = false;
    for (let n of node.subNode) {
      if (this.needShow(n)) {
        result = true;
      }
    }
    return result;
  }

  ngOnInit(): void {
    this.initClsObj();
  }

  initClsObj() {
    if (this.options && this.options.iconCls) {
      this.iconClsFolder = this.options.iconCls.folder;
      this.iconClsFile = this.options.iconCls.file;
      this.iconClsExpand = this.options.iconCls.expand;
    }
  }

  getFileIcon() {
    if (this.iconClsFile && (!this.nodeData[this.keyChild] || this.nodeData[this.keyChild].length == 0)) {
      return this.iconClsFile;
    }
    return '';
  }

  getFolderIcon() {
    if (this.iconClsFolder && this.nodeData[this.keyChild] && this.nodeData[this.keyChild].length > 0) {
      return this.iconClsFolder;
    }
    return '';
  }

  getExpandIcon() {
    if (this.iconClsExpand && this.nodeData.show) {
      return this.iconClsExpand;
    }
    return '';
  }

  clickNode(nodeData) {
    this.clickEvent.emit(this.nodeData);
  }

  changeShow() {
    this.nodeData.show = !this.nodeData.show;
    this.extendEvent.emit(this.nodeData);
  }

  doNodeFn(option) {
    if (option.fn) {
      option.fn(this.nodeData);
    }
  }

  _clickNode(event) {
    this.clickEvent.emit(event);
  }

  _extendNode(event) {
    this.extendEvent.emit(event);
  }

  ngOnChanges(changes: SimpleChanges) {
  }


  getText() {
    if (this._nodeData.show) {
      return '-';
    } else {
      return '+';
    }
  }
}
